import pdb

def get_opiec_labels(opiec_lines):
	opiec_labels = {}
	for line in opiec_lines:
		if "#" in line:
			line_split = line.split("\t")
			if len(line_split) == 21:
				key = line_split[0]
				opiec_labels[key] = {}
				opiec_labels[key]['hit-rel'] = line_split[16][0]
				opiec_labels[key]['any-rel'] = line_split[19][0]
				opiec_labels[key]['kb-formula'] = line_split[20][0]

	return opiec_labels

def get_openie5_labels(openie5_lines):
	openie5_labels = {}
	key = "#000"
	for line in openie5_lines:
		line_split = line.split("\t")
		if "#" in line:
			key = line_split[0]
			openie5_labels[key] = "UNDEFINED"
		if len(line_split) == 5:
			openie5_labels[key] = {}
			openie5_labels[key]['hit-rel'] = line_split[2][0]
			openie5_labels[key]['any-rel'] = line_split[3][0]
			openie5_labels[key]['kb-formula'] = line_split[4][0]

	return openie5_labels

def get_rnnoie_labels(rnnoie_lines):
	rnnoie_labels = {}
	key= "#000"
	for line in rnnoie_lines:
		line_split = line.split("\t")
		if len(line_split) >= 3:
			if "#" in line:
				key = line_split[0]
				rnnoie_labels[key] = "UNDEFINED"
			else:
				rnnoie_labels[key] = {}
				rnnoie_labels[key]['hit-rel'] = line_split[-3][0]
				rnnoie_labels[key]['any-rel'] = line_split[-2][0]
				rnnoie_labels[key]['kb-formula'] = line_split[-1][0]
				#pdb.set_trace()

	return rnnoie_labels

def get_stanford_labels(stanford_lines):
	stanford_labels = {}
	key = "#000"
	for line in stanford_lines:
		line_split = line.split("\t")
		if "#" in line:
			key = line_split[0]
			stanford_labels[key] = "UNDEFINED"
		elif len(line_split) >= 3:
			stanford_labels[key] = {}
			stanford_labels[key]['hit-rel'] = line_split[-3][0]
			stanford_labels[key]['any-rel'] = line_split[-2][0]
			stanford_labels[key]['kb-formula'] = line_split[-1][0]

	return stanford_labels

def get_label_agree_stats(opiec_labels, openie_labels, openie_system):
	openie_stats = {}
	
	openie_stats['hit-rel'] = {}
	openie_stats['hit-rel']['lbl_agree'] = 0
	openie_stats['hit-rel']['lbl_disagree'] = 0
	openie_stats['hit-rel']['disagreeing'] = []
	openie_stats['hit-rel']['disag_opiec->oie'] = []

	openie_stats['any-rel'] = {}
	openie_stats['any-rel']['lbl_agree'] = 0
	openie_stats['any-rel']['lbl_disagree'] = 0
	openie_stats['any-rel']['disagreeing'] = []
	openie_stats['any-rel']['disag_opiec->oie'] = []

	openie_stats['kb-formula'] = {}
	openie_stats['kb-formula']['lbl_agree'] = 0
	openie_stats['kb-formula']['lbl_disagree'] = 0
	openie_stats['kb-formula']['disagreeing'] = []
	openie_stats['kb-formula']['disag_opiec->oie'] = []

	# Hit relations
	for key in opiec_labels:
		if openie_labels[key]['hit-rel'] != "/":
			if openie_labels[key]['hit-rel'] == opiec_labels[key]['hit-rel']:
				openie_stats['hit-rel']['lbl_agree'] += 1
			else:
				openie_stats['hit-rel']['lbl_disagree'] += 1
				openie_stats['hit-rel']['disagreeing'].append(key + "-" + openie_system + ":opiec_label=" + opiec_labels[key]['hit-rel'] + "; " + openie_system + "_label=" + openie_labels[key]['hit-rel'])
				openie_stats['hit-rel']['disag_opiec->oie'].append(opiec_labels[key]['hit-rel'] + "->" + openie_labels[key]['hit-rel'])
	
	# Any relations
	for key in opiec_labels:
		if openie_labels[key]['any-rel'] != "/":
			if openie_labels[key]['any-rel'] == opiec_labels[key]['any-rel']:
				openie_stats['any-rel']['lbl_agree'] += 1
			else:
				openie_stats['any-rel']['lbl_disagree'] += 1
				openie_stats['any-rel']['disagreeing'].append(key + "-" + openie_system + ":opiec_label=" + opiec_labels[key]['any-rel'] + "; " + openie_system + "_label=" + openie_labels[key]['any-rel'])
				openie_stats['any-rel']['disag_opiec->oie'].append(opiec_labels[key]['any-rel'] + "->" + openie_labels[key]['any-rel'])

	# kb formulas
	for key in opiec_labels:
		if openie_labels[key]['kb-formula'] != "/":
			if openie_labels[key]['kb-formula'] == opiec_labels[key]['kb-formula']:
				openie_stats['kb-formula']['lbl_agree'] += 1
			else:
				openie_stats['kb-formula']['lbl_disagree'] += 1
				openie_stats['kb-formula']['disagreeing'].append(key + "-" + openie_system + ":opiec_label=" + opiec_labels[key]['kb-formula'] + "; " + openie_system + "_label=" + openie_labels[key]['kb-formula'])
				openie_stats['kb-formula']['disag_opiec->oie'].append(opiec_labels[key]['kb-formula'] + "->" + openie_labels[key]['kb-formula'])

	return openie_stats

def get_all_labeling_stats(openie5_stats, rnnoie_stats, stanford_stats):
	all_stats = {}
	all_stats['hit-rel'] = {}
	all_stats['any-rel'] = {}
	all_stats['kb-formula'] = {}
	
	all_stats['hit-rel']['lbl_agree'] = openie5_stats['hit-rel']['lbl_agree'] + rnnoie_stats['hit-rel']['lbl_agree'] + stanford_stats['hit-rel']['lbl_agree']
	all_stats['hit-rel']['lbl_disagree'] = openie5_stats['hit-rel']['lbl_disagree'] + rnnoie_stats['hit-rel']['lbl_disagree'] + stanford_stats['hit-rel']['lbl_disagree']
	all_stats['hit-rel']['disagreeing'] = openie5_stats['hit-rel']['disagreeing'] + rnnoie_stats['hit-rel']['disagreeing'] + stanford_stats['hit-rel']['disagreeing']
	all_stats['hit-rel']['disag_opiec->oie'] = openie5_stats['hit-rel']['disag_opiec->oie'] + rnnoie_stats['hit-rel']['disag_opiec->oie'] + stanford_stats['hit-rel']['disag_opiec->oie']

	all_stats['any-rel']['lbl_agree'] = openie5_stats['any-rel']['lbl_agree'] + rnnoie_stats['any-rel']['lbl_agree'] + stanford_stats['any-rel']['lbl_agree']
	all_stats['any-rel']['lbl_disagree'] = openie5_stats['any-rel']['lbl_disagree'] + rnnoie_stats['any-rel']['lbl_disagree'] + stanford_stats['any-rel']['lbl_disagree']
	all_stats['any-rel']['disagreeing'] = openie5_stats['any-rel']['disagreeing'] + rnnoie_stats['any-rel']['disagreeing'] + stanford_stats['any-rel']['disagreeing']
	all_stats['any-rel']['disag_opiec->oie'] = openie5_stats['any-rel']['disag_opiec->oie'] + rnnoie_stats['any-rel']['disag_opiec->oie'] + stanford_stats['any-rel']['disag_opiec->oie']

	all_stats['kb-formula']['lbl_agree'] = openie5_stats['kb-formula']['lbl_agree'] + rnnoie_stats['kb-formula']['lbl_agree'] + stanford_stats['kb-formula']['lbl_agree']
	all_stats['kb-formula']['lbl_disagree'] = openie5_stats['kb-formula']['lbl_disagree'] + rnnoie_stats['kb-formula']['lbl_disagree'] + stanford_stats['kb-formula']['lbl_disagree']
	all_stats['kb-formula']['disagreeing'] = openie5_stats['kb-formula']['disagreeing'] + rnnoie_stats['kb-formula']['disagreeing'] + stanford_stats['kb-formula']['disagreeing']
	all_stats['kb-formula']['disag_opiec->oie'] = openie5_stats['kb-formula']['disag_opiec->oie'] + rnnoie_stats['kb-formula']['disag_opiec->oie'] + stanford_stats['kb-formula']['disag_opiec->oie']

	return all_stats

def compute_agreement_all_relations():
	# Read lines of the files
	opiec_lines = open("ota_opiec_labeled.txt", "r").readlines()
	openie5_lines = open("ota_openie5_oie_filtered.txt", "r").readlines()
	rnnoie_lines = open("ota_rnnoie_oie_filtered.txt", "r").readlines()
	stanford_lines = open("ota_stanford_oie_filtered.txt", "r").readlines()

	# Assign labels to each OIE system
	opiec_labels = get_opiec_labels(opiec_lines)
	openie5_labels = get_openie5_labels(openie5_lines)
	rnnoie_labels = get_rnnoie_labels(rnnoie_lines)
	stanford_labels = get_stanford_labels(stanford_lines)

	# Get OIE system's agreement stats
	openie5_stats = get_label_agree_stats(opiec_labels, openie5_labels, "OpenIE-5")
	rnnoie_stats = get_label_agree_stats(opiec_labels, rnnoie_labels, "RnnOIE")
	stanford_stats = get_label_agree_stats(opiec_labels, stanford_labels, "Stanford")

	# Merge all stats
	all_stats = get_all_labeling_stats(openie5_stats, rnnoie_stats, stanford_stats)
	
	pdb.set_trace()

def main():
	compute_agreement_all_relations()

if __name__ == '__main__':
	main()